﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace ExtraltodeusPlugin {
	public class CameraRigPlugin : MVRScript {
		private Dictionary<string, UIDynamicToggle> toggles = new Dictionary<string, UIDynamicToggle>();

		private JSONStorableFloat X;
		private JSONStorableFloat Y;
		private JSONStorableFloat Z;

		private JSONStorableFloat Xa;
		private JSONStorableFloat Ya;
		private JSONStorableFloat Za;

		private JSONStorableStringChooser chooser;

		public override void Init() {
			try
			{
				X = new JSONStorableFloat("Position X adjustment", -0.1f, -1.0f, 1.0f, true);
				X.storeType = JSONStorableParam.StoreType.Full;
				CreateSlider(X);
				RegisterFloat(X);

				Y = new JSONStorableFloat("Position Y adjustment", 0.1f, -1.0f, 1.0f, true);
				Y.storeType = JSONStorableParam.StoreType.Full;
				CreateSlider(Y);
				RegisterFloat(Y);

				Z = new JSONStorableFloat("Position Z adjustment", -0.1f, -1.0f, 1.0f, true);
				Z.storeType = JSONStorableParam.StoreType.Full;
				CreateSlider(Z);
				RegisterFloat(Z);

				Xa = new JSONStorableFloat("Rotation X adjustment", 0f, -1.0f, 1.0f, true);
				Xa.storeType = JSONStorableParam.StoreType.Full;
				CreateSlider(Xa);
				RegisterFloat(Xa);

				Ya = new JSONStorableFloat("Rotation Y adjustment", 0f, -1.0f, 1.0f, true);
				Ya.storeType = JSONStorableParam.StoreType.Full;
				CreateSlider(Ya);
				RegisterFloat(Ya);

				Za = new JSONStorableFloat("Rotation Z adjustment", 0f, -1.0f, 1.0f, true);
				Za.storeType = JSONStorableParam.StoreType.Full;
				CreateSlider(Za);
				RegisterFloat(Za);

				List<string> choices = new List<string>();
				choices.Add("Update");
				choices.Add("FixedUpdate");
				chooser = new JSONStorableStringChooser("Update freq", choices, "Update", "Update frequency");
				CreatePopup(chooser, true);

				JSONStorableBool NoVertical = new JSONStorableBool("NoVertical", false);
				NoVertical.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(NoVertical);
				toggles["NoVertical"] = CreateToggle((NoVertical), true);

				JSONStorableBool playingBool = new JSONStorableBool("Play", true);
				playingBool.storeType = JSONStorableParam.StoreType.Full;
				RegisterBool(playingBool);
				toggles["Play"] = CreateToggle((playingBool), true);
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void animate(){
			if (toggles["Play"].toggle.isOn)
			{
				Vector3 pos = SuperController.singleton.lookCamera.transform.position;
				Vector3 rot = SuperController.singleton.lookCamera.transform.eulerAngles;

				pos[0] = X.val;
				pos[1] = Y.val;
				pos[2] = Z.val;

				rot[0] = rot[0] + Xa.val*90f;
				rot[1] = rot[1] + Ya.val*90f;
				rot[2] = rot[2] + Za.val*90f;

				pos = SuperController.singleton.lookCamera.transform.TransformPoint(pos);
				if (toggles["NoVertical"].toggle.isOn)
					pos[1] = containingAtom.GetStorableByID("control").transform.position.y;

				containingAtom.GetStorableByID("control").transform.position    = pos;
				containingAtom.GetStorableByID("control").transform.eulerAngles = rot;
			}
		}

		void FixedUpdate() {
			try
			{
				if (chooser.val == "FixedUpdate"){
					animate();
				}
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void Update() {
			try
			{
				if (chooser.val == "Update"){
					animate();
				}
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}
	}
}
